function res = example_neuralNetwork_transformer_medical_queries_zono()
% example_neuralNetwork_transformer_medical_queries_zono - example for the verification of a 
%    transformer neural network from 
%
% Syntax:
%    res = example_neuralNetwork_transformer_medical_queries_zono()
%
% Inputs:
%    -
%
% Outputs:
%    res - boolean
%
% References:
%    [1] Vaswani et al. "Attention is all you need", 2017
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% See also: -

% Authors:       Rayen Mhadhbi
% Written:       05-Sept-2024
% Last update:   ---
% Last revision: ---

% ------------------------------ BEGIN CODE -------------------------------

% load nn
verbose = false;
nn_cora = neuralNetwork.readTransformerNetwork('/Users/rayenmhadhbi/PycharmProjects/BachelorArbeit/transformer_model_in_medical_queries_big_snd_modal.json', verbose);
display(nn_cora)
mn = min(nn_cora.layers{1, 1}.token_emb(:));
mx = max(nn_cora.layers{1, 1}.token_emb(:));
display(mn);
display(mx);

% define the input ( tokenized sentence : How to deal with tired eyes and
% awake brain)
input = [ 101, 2129, 2000, 3066, 2007, 5458, 2159, 1998, 8300, 4167,  102, ...
         0];

% Make predictions
pred_1 = nn_cora.layers{1, 1}.evaluate(input);
pred_2 = nn_cora.layers{2, 1}.evaluate(pred_1);
pred_3 = nn_cora.layers{3, 1}.evaluate(pred_2);
pred_4 = nn_cora.layers{4, 1}.evaluate(pred_3);
pred_5 = nn_cora.layers{5, 1}.evaluate(pred_4);
pred_6 = nn_cora.layers{6, 1}.evaluate(pred_5);
pred_7 = nn_cora.layers{7, 1}.evaluate(pred_6);
 pred_8 = nn_cora.layers{8, 1}.evaluate(pred_7);

[~, label] = max(pred_8);
fprintf("Most likely label is: %d\n", label-1);

% define embedding noise range
noise_range = logspace(-8, log10(0.001), 100);  
radii = zeros(size(noise_range));
isVerifiedArray = false(size(noise_range));


for i = 1:length(noise_range)
    noise = 0.05;
    embedding = nn_cora.layers{1, 1}.evaluate(input);
    c = reshape(embedding, [], 1); 
    dim = length(c);
    G = noise * eye(dim);
    z = zonotope(c, G);
    options = struct();
    options.nn.num_generators = 1000;
    options.nn.add_approx_error_to_GI = true;
    options.nn.approach_transformer = "zonotope_precise";
    options = nnHelper.validateNNoptions(options);

     z = nn_cora.layers{2, 1}.evaluate(z, options);
    int = interval(z);
    inf = reshape(int.inf, 12, 8);
    sup = reshape(int.sup, 12, 8);
    int = interval(inf, sup);
    z = nn_cora.layers{3, 1}.evaluate(z, options);
    int = interval(z);
    inf = reshape(int.inf, 12, 8);
    sup = reshape(int.sup, 12, 8);
    int = interval(inf, sup);
    z = nn_cora.layers{4, 1}.evaluate(z, options);
    int = interval(z);
    z = nn_cora.layers{5, 1}.evaluate(z, options);
    int = interval(z);
    z = nn_cora.layers{6, 1}.evaluate(z, options);
    int = interval(z);
    z = nn_cora.layers{7, 1}.evaluate(z, options);
    int = interval(z);
    inf = reshape(int.inf, 2, 1);
    sup = reshape(int.sup, 2, 1);
    int = interval(inf, sup);
    
    M = [1 -1; 0 0];
    inf = M * int.inf;
    sup = M * int.sup; 
    r = radius(M* int);
    radii(i) = r;
   

    label_pred = interval(inf(label), sup(label));
    other_pred = interval(min(sup(2-label+1),inf(2-label+1)), max(sup(2-label+1),inf(2-label+1)));

    isVerified = all(other_pred.sup < label_pred.inf);
    isVerifiedArray(i) = isVerified;  
end
numVerified = sum(isVerifiedArray);
fprintf('Number of verified entries: %d\n', numVerified);
figure;
hold on;
grid on;
xlabel('Embedding Noise');
ylabel('Radius of Interval enclosure of pZ (only for attention block)');
title('Certification Radii for pZ Propagation');

% plot verified radii in one color (e.g., blue)
semilogx(noise_range(isVerifiedArray), radii(isVerifiedArray), 'bo', 'MarkerFaceColor', 'b');

% plot non-verified radii in another color (e.g., red)
semilogx(noise_range(~isVerifiedArray), radii(~isVerifiedArray), 'ro', 'MarkerFaceColor', 'r');

legend('Verified Radii', 'Non-verified Radii');
hold off;

res = true;
end

% ------------------------------ END OF CODE ------------------------------
